// Untidy.NantTasks is Copyright (c) 2002 Simon Steele
// All rights reserved.
//
// See license.txt for the license conditions (BSD Style)

using System;
using NAnt.Core;
using NAnt.Core.Tasks;
using NAnt.Core.Attributes;

namespace Untidy.NAntTasks
{
	/// <summary>
	/// Summary description for MKSSITask.
	/// </summary>
	[TaskName("mkssi")]
	public class MKSSITask : ExternalProgramBase
	{
		enum MKSOperation
		{
			sandbox
		}

		enum MKSVolume
		{
			normal,
			quiet,
			terse
		}

		private MKSOperation _op = MKSOperation.sandbox;
		private MKSVolume _vol = MKSVolume.normal;
        private string _sandboxProjectFile = "sandbox.pj";
		private string _sandboxDir = null;
		private string _mksProject = null;
		private string _params = null;
		private string _revision = null;
		private bool _recursive = true;

		/// <summary>Directory in which the sandbox will be placed.</summary>
		[TaskAttribute("dir", Required=true)]
		public string Dir
		{
			get { return _sandboxDir; }
			set 
			{ 
				_sandboxDir = value; 
				if(_sandboxDir != null)
					if(_sandboxDir[_sandboxDir.Length - 1] != '\\')
						_sandboxDir += '\\';
			}
		}

		/// <summary>Project file (.pj) to work with.</summary>
		[TaskAttribute("project", Required=true)]
		public string MKSProject
		{
			get { return _mksProject; }
			set { _mksProject = value; }
		}

		/// <summary>Revision to work with.</summary>
		[TaskAttribute("revision", Required=false)]
		public string Revision
		{
			get { return _revision; }
			set { _revision = value; }
		}

		/// <summary>Recursive (true/false)</summary>
		[TaskAttribute("recursive", Required=false)]
		public bool Recursive
		{
			get { return _recursive; }
			set { _recursive = value; }
		}

		/// <summary>Quiet (true/false)</summary>
		[TaskAttribute("quiet", Required=false)]
		public bool Quiet
		{
			get { return (_vol == MKSVolume.quiet); }
			set 
			{ 
				if(value) 
					_vol = MKSVolume.quiet; 
				else if(!value && _vol == MKSVolume.quiet) 
					_vol = MKSVolume.normal; 
			}
		}

		/// <summary>Very quiet (true/false)</summary>
		[TaskAttribute("terse", Required=false)]
		public bool Terse
		{
			get { return (_vol == MKSVolume.terse); }
			set 
			{ 
				if(value) 
					_vol = MKSVolume.terse; 
				else if(!value && _vol == MKSVolume.terse) 
					_vol = MKSVolume.normal; 
			}
		}

		public override string ProgramFileName
		{
			get	{ return "pj"; }
		}

		public override string ProgramArguments 
		{
			get 
			{
				if(_params == null)
					throw new BuildException("Build parameters have not been initialised.");

				return _params;
			}
		}

		private string GetGeneralParameters()
		{
			string p = "";

			if(_vol == MKSVolume.quiet)
				p += " -q";
			else if(_vol == MKSVolume.terse)
				p += " -Q";

			if(_recursive)
				p += " -i";

			if(_revision != null)
				p += " -k " + _revision;

			// Add the master project parameter... (-P "MKSRoot\\master_project.pj")
			p += " -P \"" + _mksProject + "\"";

			// Add the sandbox parameter... (-S "c:\source\myproject\sandbox.pj")
			p += " -S \"" + _sandboxDir + _sandboxProjectFile + "\"";

			return p;
		}

		protected override void InitializeTask(System.Xml.XmlNode taskNode) 
		{

			if(_sandboxDir == null)
				throw new BuildException("Dir is a required attribute.", Location);

			if(_mksProject == null)
				throw new BuildException("Project is a required attribute.", Location);

			string p = "";

			switch(_op)
			{
				case MKSOperation.sandbox:
					p = "sandbox";
					break;
			}

			p += " ";

			string generalopts = GetGeneralParameters();
			p += generalopts;

			_params = p;
		}

		protected override void ExecuteTask() 
		{
			if(_params != null)
				Log(Level.Info, LogPrefix + _params);

			base.ExecuteTask();
		}
	}
}
