// Untidy.NantTasks is Copyright (c) 2002 Simon Steele
// All rights reserved.
//
// See license.txt for the license conditions (BSD Style)

using System;
using System.Text.RegularExpressions;

using NAnt.Core;

namespace Untidy.NAntTasks
{
	public enum VersionInfoTypes
	{
		FileVersion,
		ProductVersion,
		Both
	}
	
	public class VersionInfoFile
	{
		// These are the regular expressions used to match different types of version number string.
		Regex reFileVersion = new Regex("(FILEVERSION )((?<a>[0-9]+),[ ]*(?<b>[0-9]+),[ ]*(?<c>[0-9]+),[ ]*(?<d>[0-9]+))");
		Regex reProductVersion = new Regex("(PRODUCTVERSION )((?<a>[0-9]+),[ ]*(?<b>[0-9]+),[ ]*(?<c>[0-9]+),[ ]*(?<d>[0-9]+))");
		Regex reFileVersionS = new Regex("(VALUE \"FileVersion\", \")((?<a>[0-9]+)[\\.,](?<b>[0-9]+)[\\.,](?<c>[0-9]+)[\\.,](?<d>[0-9]+))(?<end>\")");
		Regex reFileVersionS2 = new Regex("(VALUE \"FileVersion\", \")([a-zA-Z0-9-_\\.,]+)(?<end>\")");
		Regex reProductVersionS = new Regex("(VALUE \"ProductVersion\", \")((?<a>[0-9]+)[\\.,](?<b>[0-9]+)[\\.,](?<c>[0-9]+)[\\.,](?<d>[0-9]+))(?<end>\")");
		Regex reProductVersionS2 = new Regex("(VALUE \"ProductVersion\", \")([a-zA-Z0-9-_\\.,]+)(?<end>\")");

		VersionInfoTypes _target = VersionInfoTypes.Both;
		private string _productSuffix = "";
		private string _fileSuffix = "";
		private string _filename = "";

		public VersionInfoTypes Target
		{
			set { _target = value; }
		}

		public string ProductSuffix
		{
			set { _productSuffix = value; }
		}

		public string FileSuffix
		{
			set { _fileSuffix = value; }
		}

		public VersionInfoFile()
		{

		}

		public VersionInfoFile(string fileName)
		{
			_filename = fileName;
		}

		/// <summary>
		/// This searches for a version number in "filename" and returns a 
		/// VersionNumber instance on success.
		/// </summary>
		public bool ReadVersion(VersionInfoTypes type, VersionNumber version)
		{
			if(type == VersionInfoTypes.Both)
				throw new BuildException("Can only read from FileVersion or ProductVersion - not both.");
			
			System.IO.FileInfo fi = new System.IO.FileInfo(_filename);
			if(!fi.Exists)
				throw new BuildException("Cannot read from non-existent file: " + _filename);

			bool bSuccess = false;

			Regex matchregex = (type == VersionInfoTypes.FileVersion) ? reFileVersion : reProductVersion;
			Match m = null;
			SimpleTextFile fIn = new SimpleTextFile(_filename);

			fIn.Open();
			
			while(!fIn.EOF())
			{
				m = matchregex.Match(fIn.ReadLine());
				if(m.Success)
				{
					bSuccess = true;
					version.SetFromMatch(m);
					break;
				}
			}

			fIn.Close();
			
			return bSuccess;
		}

		/// <summary>
		/// This function updates every version information 
		/// reference in the VERSIONINFO block, according to the target
		/// set in the class.
		/// </summary>
		public void UpdateVersion(SimpleTextFile infile, SimpleTextFile outfile, VersionNumber vi)
		{
			string line;

			while(!infile.EOF())
			{
				line = infile.ReadLine();

				if(_target == VersionInfoTypes.FileVersion || _target == VersionInfoTypes.Both)
				{
					line = vi.SwapVersionTags(line, reFileVersion, true, "");
					line = vi.SwapVersionTags(line, reFileVersionS2, false, _fileSuffix);
				}				
				if(_target == VersionInfoTypes.ProductVersion || _target == VersionInfoTypes.Both)
				{
					line = vi.SwapVersionTags(line, reProductVersion, true, "");
					line = vi.SwapVersionTags(line, reProductVersionS2, false, _productSuffix);
				}

				outfile.WriteLine(line);
			}
		}

		public UpdateFunction GetUpdateFunction()
		{
			return new UpdateFunction(UpdateVersion);
		}
	}

} // namespace Untidy.NAntTasks