// Untidy.NantTasks is Copyright (c) 2002 Simon Steele
// All rights reserved.
//
// See license.txt for the license conditions (BSD Style)

using System;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;

using NAnt.Core;
using NAnt.Core.Attributes;

namespace Untidy.NAntTasks
{
	

	/// <summary>
	/// VERSIONINFO Version Number Update Task
	/// </summary>
	/// <remarks>
	///   <para>This task can update a VERSIONINFO block inside 
	///   (for example) .rc or .rc2 files in order to increment/modify/replace 
	///   the version number inside.</para>
	/// </remarks>
	/// <example>
	///   <para>Increment the build number in <c>sample.rc</c>.</para>
	///   <code>
	///     &lt;versioninfo filename="sample.rc" &gt;
	///       &lt;versionnumber source="VersionInfoFile" file="sample.rc" build="1" /&gt;
	///     &lt;/versioninfo&gt;
	///   </code>
	///   <para>Load the version number from the file "build.number.file", increment the build 
	///   part and store it in both "myproduct.rc" and "build.number.file".</para>
	///   <code>&lt;versioninfo filename="myproduct.rc" source="build.number.file" /&gt;</code>
	///   <para>Increment the build part, and set the revision number to 13.</para>
	///   <code>&lt;versioninfo filename="myproduct.rc" setrevision="13" /&gt;</code>
	///   <para>Increment the minor part by 2, and don't increment the build part.</para>
	///   <code>&lt;versioninfo filename="myproduct.rc" minor="2" build="0" /&gt;</code>
	///   <para>Set the version number to 1.2.3.4.</para>
	///   <code>&lt;versioninfo filename="myproduct.rc" setversion="1.2.3.4" /&gt;</code>
	///   <para>Increment the build number by one, and add the suffix of "-devel" to the 
	///   ProductVersion string entry. This line only modifies the ProductVersion entries.</para>
	///   <code>&lt;versioninfo filename="myproduct.rc" productsuffix="-devel" update="ProductVersion" /&gt;</code>
	/// </example>
	[TaskName("versioninfo")]
	public class VersionInfoTask : VersionNumberUpdater
	{
		VersionInfoFile vif = new VersionInfoFile();

		#region Task Attributes

		/// <summary>
		/// Update File Version, Product Version or Both.
		/// </summary>
		/// <remarks>
		/// Possible values are FileVersion, ProductVersion or Both.
		/// </remarks>
		[TaskAttribute("type", Required=false)]
		public string UpdateWhat
		{
			set { vif.Target = (VersionInfoTypes)Enum.Parse(typeof(VersionInfoTypes), value); }
		}

		/// <summary>
		/// The string value representing the product version number can be suffixed with this string.
		/// </summary>
		[TaskAttribute("productsuffix", Required=false)]
		public string ProductSuffix
		{
			set { vif.ProductSuffix = value; }
		}

		/// <summary>
		/// The string value representing the file version number can be suffixed with this string.
		/// </summary>
		[TaskAttribute("filesuffix", Required=false)]
		public string FileSuffix
		{
			set { vif.FileSuffix = value; }
		}

		#endregion

		protected override void InitializeTask(System.Xml.XmlNode taskNode) 
		{
			base.InitializeTask(taskNode);
		}

		protected override void ExecuteTask()
		{
			// As this is an updater, we check that the file in question exists.
			checkFile();

			// Now we write to the file using the update function from VersionInfoFile.
			updateFile( vif.GetUpdateFunction() );
		}
	}
}
