// Untidy.NantTasks is Copyright (c) 2002 Simon Steele
// All rights reserved.
//
// See license.txt for the license conditions (BSD Style)

using System;
using System.Text;
using System.Text.RegularExpressions;

using NAnt.Core;
using NAnt.Core.Attributes;

namespace Untidy.NAntTasks
{
	public enum NumberSources
	{
		None,
		VersionInfoFile,
		VersionNumberFile,
		RegExFile
	}

	/// <summary>
	/// A class representing a four part version number (major.minor.revision.build).
	/// </summary>
	public class VersionNumber
	{
		public int major;
		public int minor;
		public int revision;
		public int build;

		public bool _useCommas = false;

		public VersionNumber()
		{
			major = 1;
			minor = revision = build = 0;
		}

		public VersionNumber(int major_, int minor_, int revision_, int build_)
		{
			major = major_;
			minor = minor_;
			revision = revision_;
			build = build_;
		}

		public override String ToString()
		{
			string res;

			if(_useCommas)
			{
				res = string.Format("{0},{1},{2},{3}", major, minor, revision, build);
			}
			else
			{
				res = string.Format("{0}.{1}.{2}.{3}", major, minor, revision, build);
			}

			return res;
		}

		public string ToString(string format)
		{
			return string.Format(format, major, minor, revision, build);
		}

		public string ToString(string format, string suffix)
		{
			return string.Format(format, major, minor, revision, build, suffix);
		}

		/// <summary>This function fills in a VersionNumber class with values from a RegEx match.</summary>
		/// <param name="match">RegEx match with a,b,c and d named values for major.minor.revision.build.</param>
		public void SetFromMatch(Match match)
		{
			major	= Int32.Parse(match.Groups["a"].ToString());
			minor	= Int32.Parse(match.Groups["b"].ToString());
			revision = Int32.Parse(match.Groups["c"].ToString());
			build	= Int32.Parse(match.Groups["d"].ToString());
		}

		/// <summary>This function fills in a VersionNumber class with values from a RegEx match.</summary>
		/// <param name="match">RegEx match with a,b,c and d named values for major.minor.revision.build.</param>
		/// <returns>VersionNumber class with values from the RegEx match.</returns>
		public static VersionNumber ExtractVersionNumber(Match match)
		{
			VersionNumber vn = new VersionNumber();
			vn.SetFromMatch(match);
			return vn;
		}

		public static string MakeFormatString(string s)
		{
			string s2 = s.Replace("M", "{0}");
			s2 = s2.Replace("m", "{1}");
			s2 = s2.Replace("r", "{2}");
			s2 = s2.Replace("b", "{3}");
			s2 = s2.Replace("s", "{4}");
			return s2;
		}

		/// <summary>
		/// If the regular expression (matcher) succeeds, then the version number 
		/// string is replaced with a new one.
		/// </summary>
		/// <param name="line">String to search.</param>
		/// <param name="matcher">Regular expression.</param>
		/// <param name="vi">New version number.</param>
		/// <param name="useCommas">Use commas or full-stops to separate.</param>
		/// <param name="suffix">Any string suffix required.</param>
		/// <returns>Updated string.</returns>
		public string SwapVersionTags(string line, Regex matcher, bool useCommas, string suffix)
		{
			Match m = matcher.Match(line);
			if(m.Success)
			{
				_useCommas = useCommas;

				string replacestr = string.Format("${{1}}{0}{1}", this.ToString(), suffix);
				if(m.Groups["end"].Length != 0)
					replacestr += "${end}";

				string ret = matcher.Replace(line, replacestr);
				return ret;
			}

			return line;
		}
	}

	/// <remarks>
	///   <list type="bullet">
	///     <item>All parts of the version number can all be incremented/decremented/replaced.</item>
	///     <item>The entire version number can be replaced with a specific new value.</item>
	///     <item>The initial version number can be read from an external file with 
	///     a version number string in it (which can also be updated).</item>
	///     <item>Suffixes can be placed on the end of the version number strings for File or 
	///     Product entries.</item>
	///   </list>
	///   <note>All build numbers are assumed to be in the form major.minor.revision.build</note>
	/// </remarks>
	[ElementName("versionnumber")]
	public class VersionNumberElement : DataTypeBase
	{
		enum VISetBits
		{
			Major = 1,
			Minor = 2,
			Revision = 4,
			Build = 8
		}

		Regex reVersionStr = new Regex(@"(?<a>[0-9]+)[\.,](?<b>[0-9]+)[\.,](?<c>[0-9]+)[\.,](?<d>[0-9]+)");
		
		VersionNumber		_version = new VersionNumber();
		VersionNumber		_incNumbers = new VersionNumber();
		VersionNumber		_setNumbers = new VersionNumber();

		NumberSources		_source = NumberSources.None;
		VersionInfoTypes	_vitype = VersionInfoTypes.ProductVersion;
		int					_setMask = 0;
		string				_filename = null;
		string				_property = null;

		#region Increment Attributes

		/// <summary>Increments the major part of the version number by value.</summary>
		[TaskAttribute("major", Required=false)]
		public int Major
		{
			get { return _incNumbers.major; }
			set { _incNumbers.major = value; }
		}

		/// <summary>Increments the minor part of the version number by value.</summary>
		[TaskAttribute("minor", Required=false)]
		public int Minor
		{
			get { return _incNumbers.minor; }
			set { _incNumbers.minor = value; }
		}

		/// <summary>Increments the revision part of the version number by value.</summary>
		[TaskAttribute("revision", Required=false)]
		public int Revision
		{
			get { return _incNumbers.revision; }
			set { _incNumbers.revision = value; }
		}

		/// <summary>Increments the build part of the version number by value. Defaults to 1.</summary>
		[TaskAttribute("build", Required=false)]
		public int Build
		{
			get { return _incNumbers.build; }
			set { _incNumbers.build = value; }
		}

		#endregion
		
		#region Set Attributes

		/// <summary>Sets the major part of the version number.</summary>
		[TaskAttribute("setmajor", Required=false)]
		public int SetMajor
		{
			get { return _setNumbers.major; }
			set { _setNumbers.major = value; _setMask |= (int)VISetBits.Major; }
		}

		/// <summary>Sets the minor part of the version number.</summary>
		[TaskAttribute("setminor", Required=false)]
		public int SetMinor
		{
			get { return _setNumbers.minor; }
			set { _setNumbers.minor = value; _setMask |= (int)VISetBits.Minor; }
		}

		/// <summary>Sets the revision part of the version number.</summary>
		[TaskAttribute("setrevision", Required=false)]
		public int SetRevision
		{
			get { return _setNumbers.revision; }
			set { _setNumbers.revision = value; _setMask |= (int)VISetBits.Revision; }
		}

		/// <summary>Sets the build part of the version number.</summary>
		[TaskAttribute("setbuild", Required=false)]
		public int SetBuild
		{
			get { return _setNumbers.build; }
			set { _setNumbers.build = value; _setMask |= (int)VISetBits.Build; }
		}

		#endregion

		#region Other Attributes

		[TaskAttribute("regex")]
		public string RegEx
		{
			set
			{
				try
				{
					reVersionStr = new Regex(value);
				}
				catch( Exception ex )
				{
					throw new BuildException("Could not create regex with expression " + value, Location, ex);
				}
			}
		}

		[TaskAttribute("number")]
		public string NumberString
		{
			set
			{
				Match m = reVersionStr.Match(value);
				if(m != null && m.Success)
				{
					_version.SetFromMatch(m);
				}
				else
					throw new BuildException("Could not match string " + value, Location);
			}
		}

		[TaskAttribute("source")]
		public string Source
		{
			set { _source = (NumberSources)Enum.Parse(typeof(NumberSources), value); }
			get { return _source.ToString(); }
		}

		[TaskAttribute("file")]
		public string Filename
		{
			set { _filename = value; }
			get { return _filename; }
		}

		[TaskAttribute("type")]
		public string VIType
		{
			set { _vitype = (VersionInfoTypes)Enum.Parse(typeof(VersionInfoTypes), value); }
			get { return _vitype.ToString(); }
		}

		[TaskAttribute("property")]
		public string Property
		{
			set { _property = value; }
			get { return _property; }
		}

		#endregion

		public VersionNumberElement()
		{
			_incNumbers.major = _incNumbers.minor = _incNumbers.revision = _incNumbers.build = 0;
			_setNumbers.major = _setNumbers.minor = _setNumbers.revision = _setNumbers.build = 0;
		}

		public VersionNumber Version
		{
			get { return _version; }
		}

		/// <summary>
		/// This is the magic function which will perform any clever updating code on 
		/// a Version number. Users of the VersionNumberUpdater class need only to add
		/// functionality to this function, and all derivative updater classes will get it.
		/// </summary>
		/// <param name="vi">Incoming VersionNumber object.</param>
		/// <returns>Resulting VersionNumber object.</returns>
		protected void updateVersionInfo()
		{
			_version.major += _incNumbers.major;
			_version.minor += _incNumbers.minor;
			_version.revision += _incNumbers.revision;
			_version.build += _incNumbers.build;

			if(_setMask != 0)
			{
				// we have to set certain values.
				if((_setMask & (int)VISetBits.Major) != 0)
					_version.major = _setNumbers.major;
				if((_setMask & (int)VISetBits.Minor) != 0)
					_version.minor = _setNumbers.minor;
				if((_setMask & (int)VISetBits.Revision) != 0)
					_version.revision = _setNumbers.revision;
				if((_setMask & (int)VISetBits.Build) != 0)
					_version.build = _setNumbers.build;
			}
		}

		protected override void InitializeElement(System.Xml.XmlNode element)
		{
			base.InitializeElement (element);

			if(_source != NumberSources.None)
			{
				if(_filename == null)
					throw new BuildException("Must specify a file when source is specified.", Location);

				System.IO.FileInfo fi = new System.IO.FileInfo(_filename);
				if(!fi.Exists)
					throw new BuildException("Source file must exist.", Location);

				switch(_source)
				{
					case NumberSources.VersionNumberFile:
					{
						VersionNumberFile vnf = new VersionNumberFile(_filename);
						if(!vnf.ReadVersion(_version))
							Project.Log(Level.Warning, "Could not read version number from {0}", _filename);
					}
					break;

					case NumberSources.VersionInfoFile:
					{
						VersionInfoFile vif = new VersionInfoFile(_filename);
						if(!vif.ReadVersion(_vitype, _version))
							Project.Log(Level.Warning, "Could not read version info from {0}", _filename);
					}
					break;

					case NumberSources.RegExFile:
					{
						VersionRegExFile vrf = new VersionRegExFile(_filename);
						vrf.RegEx = reVersionStr.ToString();
						if(!vrf.ReadVersion(_version))
							Project.Log(Level.Warning, "Could not read version number from {0}", _filename);
					}
					break;
				}
			}

			updateVersionInfo();

			if(_property != null && _property != string.Empty)
			{
				if(Project.Properties.Contains(_property))
				{
					Project.Properties.SetValue(_property, _version.ToString());
				}
				else
				{
					Project.Properties.Add(_property, _version.ToString());
				}
			}
		}

	}

	/// <summary>
	/// Indicates that a property should be treated as a version number.
	/// </summary>
	[AttributeUsage(AttributeTargets.Property, Inherited=true)]
	public sealed class VersionNumberAttribute : BuildElementAttribute 
	{     
		#region Public Instance Constructors

		/// <summary>
		/// Initializes a new instance of the <see cref="FileSetAttribute" /> with the
		/// specified name.
		/// </summary>
		/// <param name="name">The name of the attribute.</param>
		public VersionNumberAttribute(string name) : base(name) 
		{        
		}      

		#endregion Public Instance Constructors
	}
}
