// Untidy.NantTasks is Copyright (c) 2002 Simon Steele
// All rights reserved.
//
// See license.txt for the license conditions (BSD Style)

using System;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using NAnt.Core;
using NAnt.Core.Attributes;

namespace Untidy.NAntTasks
{
	public delegate void UpdateFunction(SimpleTextFile infile, SimpleTextFile outfile, VersionNumber vi);

	#region SimpleTextFile Class
	public class SimpleTextFile
	{
		private string filename = "";
		private	FileStream fs =	null;
		private	StreamWriter w = null;
		private	StreamReader r = null;
		
		public SimpleTextFile(string _filename)
		{
			filename = _filename;
		}

		public void Open()
		{
			Open(false);
		}

		public void	Open(bool bWrite)
		{
			if(filename == "")
				return;

			if(bWrite)
			{
				//if (logFile.Exists)
				//	File.Delete(filePath);

				fs = new FileStream(filename, FileMode.OpenOrCreate, FileAccess.ReadWrite);
				w =	new	StreamWriter(fs);
				w.BaseStream.Seek(0, SeekOrigin.End);
			}
			else
			{
				FileInfo finfo = new FileInfo(filename);
				if (!finfo.Exists)
					return;

				fs = new FileStream(filename, FileMode.Open, FileAccess.Read);
				r = new StreamReader(fs);
				r.BaseStream.Seek(0, SeekOrigin.Begin);
			}
		}

		public void SeekStart()
		{
			fs.Seek(0, SeekOrigin.Begin);
		}

		public void	WriteLine(String input)	
		{
			if(w == null)
				return;

			w.WriteLine(input);
			w.Flush();
		}

		public string ReadLine() 
		{
			if(r == null)
				return "";

			StringBuilder output = new StringBuilder();
			output.Length =	0;

			if(r.Peek()	> -1) 
			{
				output.Append(r.ReadLine());
			}

			return output.ToString();
		}

		public void Close()
		{
			if(w != null)
				w.Close();

			if(r != null)
				r.Close();

			fs.Close();
			r = null;
			w = null;
			fs = null;
		}

		public bool	EOF()
		{
			if(r == null)
				return true;

			bool bEOF =	true;

			if(r.Peek()	> -1)
				bEOF = false;

			return bEOF;
		}
	}
	#endregion

	#region FileUpdater Class - two stage update.

	public class FileUpdater
	{
		string _fileName;
		string _tempFileName;

		public SimpleTextFile InputFile = null;
		public SimpleTextFile OutputFile = null;

		public FileUpdater(string fileName)
		{
			_fileName = fileName;
		}

		public void BeginUpdate()
		{
			_tempFileName = Path.GetTempFileName();
			
			InputFile = new SimpleTextFile(_fileName);
			OutputFile = new SimpleTextFile(_tempFileName);

			InputFile.Open();
			OutputFile.Open(true);
		}

		public void EndUpdate()
		{
			if(_fileName == null || _tempFileName == null)
				throw new Exception("You must call BeginUpdate before EndUpdate.");

			InputFile.Close();
			OutputFile.Close();

			FileInfo finfo = new FileInfo(_tempFileName);
			if(finfo.Exists)
			{
				try
				{
					System.IO.File.Delete(_fileName);
					System.IO.File.Move(_tempFileName, _fileName);
				}
				catch( IOException ex )
				{
					throw new Exception("Fatal Error: " + ex.Message, ex);
				}
			}
			else
				throw new Exception("Could not find temporary file ( " + _tempFileName + " ).");
		}
	}

	#endregion

    /// <summary>
	/// Base class for tasks which update version numbers. 
	/// Provides common parameters and things.
	/// </summary>
	public abstract class VersionNumberUpdater : Task
	{
		protected string _fileName = null;
	
		public VersionNumberUpdater()
		{
		}

		#region NAnt Task Attributes

		VersionNumberElement _version = null;

		[VersionNumber("version", Required=true)]
		public VersionNumberElement Version
		{
			set
			{
				_version = value;
			}
			
			get
			{
				return _version;
			}
		}

		/// <summary>Filename (*.rc or *.rc2) of the file to update.</summary>
		/// <remarks>Filename of the file that the task will update.</remarks>
		[TaskAttribute("filename", Required=true)]
		public string FileName
		{
			get { return _fileName; }
			set { _fileName = value; }
		}

		#endregion

		/// <summary>This function updates the file "fileName" using the delegate "fn".</summary>
		/// <param name="fileName">Name and path of the file to be updated.</param>
		/// <param name="vi">New version number.</param>
		/// <param name="fn">Delegate pointing to an update function.</param>
		public void updateFile(UpdateFunction fn)
		{
			FileUpdater update = new FileUpdater(_fileName);
			update.BeginUpdate();

			try
			{
				fn(update.InputFile, update.OutputFile, _version.Version);
			}
			finally
			{
				try
				{
					update.EndUpdate();
				}
				catch( Exception ex )
				{
					throw new BuildException("Exception caught updating file.", ex);
				}
			}
		}

		/// <summary>Check that the file referenced in the filename attribute exists.</summary>
		protected void checkFile()
		{
			System.IO.FileInfo fi = new FileInfo(_fileName);
			if(!fi.Exists)
				throw new BuildException("File " + _fileName + " does not exist.", Location);
		}

		protected override void InitializeTask(System.Xml.XmlNode taskNode)
		{
			FileInfo fi;

			if(FileName == null)
				throw new BuildException("Attribute \"filename\" is required.", Location);
			
			fi = new FileInfo(FileName);
			if(!fi.Exists)
				throw new BuildException("Version number file (" + FileName + ") does not exist, and cannot be updated.", Location);
		}
	}
}
