// Untidy.NantTasks is Copyright (c) 2002 Simon Steele
// All rights reserved.
//
// See license.txt for the license conditions (BSD Style)

using System;
using System.Text;
using System.Text.RegularExpressions;
using NAnt.Core;
using NAnt.Core.Attributes;

namespace Untidy.NAntTasks
{
	public class VersionRegExFile
	{
		string _filename = null;

		Regex _reToReplace = new Regex("[0-9]+\\.[0-9]+\\.[0-9]+\\.[0-9]+");
		string _outputFormat = "M.m.r.bs";
		string _suffix = "";

		public VersionRegExFile()
		{

		}

		public VersionRegExFile(string filename)
		{
			_filename = filename;
		}

		public string RegEx
		{
			get { return _reToReplace.ToString(); }
			set { _reToReplace = new Regex(value); }
		}

		public string Format
		{
			get { return VersionNumber.MakeFormatString(_outputFormat); }
			set { _outputFormat = value; }
		}

		public string Suffix
		{
			get { return _suffix; }
			set { _suffix = value; }
		}

		public bool ReadVersion(VersionNumber version)
		{
			bool bSuccess = false;
			SimpleTextFile fIn = new SimpleTextFile(_filename);
			fIn.Open();

			if(!fIn.EOF())
			{
				Match m = _reToReplace.Match(fIn.ReadLine());
				if(m.Success)
				{
					bSuccess = true;
					version.SetFromMatch(m);
				}
				fIn.Close();
			}

			return bSuccess;
		}

		void UpdateVersion(SimpleTextFile fIn, SimpleTextFile fOut, VersionNumber vout)
		{
			string	line;
			Match	m = null;

			while(!fIn.EOF())
			{
				line = fIn.ReadLine();
				m = _reToReplace.Match(line);
				if(m.Success)
				{
					Group g = null;
					if(m.Groups.Count == 2)
					{
						//Group style replacement...
						g = m.Groups[1];
					}
					else if(m.Groups.Count == 1)
					{
						// Simple text replacement...
						g = m;
					}

					if(g == null)
						throw new BuildException("Unable to replace version number, regex does not match the correct number of groups.");

					string temp = line.Substring(0, g.Index);
					temp += vout.ToString(Format, _suffix);
					temp += line.Substring(g.Index + g.Length);
					line = temp;
				}
				
				fOut.WriteLine(line);
			}
		}

		public UpdateFunction GetUpdateFunction()
		{
			return new UpdateFunction(UpdateVersion);
		}
	}

	/// <summary>
	/// Summary description for VersionHeaderTask.
	/// </summary>
	[TaskName("versionregex")]
	public class VersionRegExTask : VersionNumberUpdater
	{
		public VersionRegExTask()
		{
		}

		VersionRegExFile file = new VersionRegExFile();

		/// <summary>
		/// This attribute sets the regular expression which will mark 
		/// out the version number to be replaced. This can take one of two
		/// forms:
		/// The first form would simply represent the entire version number:
		/// [0-9]+\.[0-9]+\.[0-9]+\.[0-9]+ or [0-9\.]+
		/// The second form marks out part of a specific string to replace:
		/// #define myproduct_version "([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+)"
		/// Note: with the second form the actual number must be grouped with brackets. 
		/// Any brackets in the string must be escaped using the backslash (\) character.
		/// 
		/// Default: [0-9]+\.[0-9]+\.[0-9]+\.[0-9]+
		/// </summary>
		[TaskAttribute("regex", Required=false)]
		public string RegEx
		{
			get { return file.RegEx; }
			set { file.RegEx = value; }
		}

		/// <summary>
		/// By default, this task writes out the version number in this format:
		/// major.minor.revision.build[-suffix]
		/// If you wish to output the version number in another form, you can 
		/// set this attribute to a string value using special characters. e.g.
		/// M.m.r-s
		/// M is replaced with major, m with minor, r with revision, b with build 
		/// and s is replaced with a suffix.
		/// </summary>
		[TaskAttribute("format", Required=false)]
		public string Format
		{
			get { return file.Format; }
			set { file.Format = value; }
		}

		/// <summary>Suffix string to use in the updated version number.</summary>
		[TaskAttribute("suffix", Required=false)]
		public string Suffix
		{
			get { return file.Suffix; }
			set { file.Suffix = value; }
		}

		protected override void ExecuteTask()
		{
			checkFile();
			updateFile(file.GetUpdateFunction());
		}
	}
}
